@@ -10,7 +10,7 @@ logging.basicConfig(
 )
 logger = logging.getLogger(__name__)
 
-from .interface import Lister, Getter, Creator, Updater, Deleter, S3, Utils
+from .interface import Lister, Getter, Creator, Updater, Deleter, S3, Utils, Node
 from .gen import MessageGenerator
 
 
@@ -90,6 +90,7 @@ class LogQSClient:
         self.s3 = S3(creator=self.create)
         self.utils = Utils(s3=self.s3, config=self.config)
         self.gen = MessageGenerator(getter=self.get, lister=self.list)
+        self.node = Node(getter=self.get, lister=self.list, config=self.config)
 
 
 LogQS = LogQSClient
