@@ -4,6 +4,7 @@ from typing import Optional, Union, List
 
 import requests
 import xmltodict
+from urllib.parse import unquote
 
 
 class S3Method(str, Enum):
@@ -112,6 +113,7 @@ class S3:
         upload_id: str,
         parts: List[dict],
         convert_to_dict: bool = True,
+        decode_response: bool = True,
     ):
         if resource in ["record"]:
             if (
@@ -151,7 +153,13 @@ class S3:
         complete_multipart_payload += "</CompleteMultipartUpload>"
 
         r = requests.post(url, data=complete_multipart_payload)
-        response_content = xmltodict.parse(r.text) if convert_to_dict else r.text
+        if decode_response:
+            response_text = unquote(r.text)
+        else:
+            response_text = r.text
+        response_content = (
+            xmltodict.parse(response_text) if convert_to_dict else response_text
+        )
         return r.headers, used_params, response_content
 
     def create_multipart_upload(
@@ -160,6 +168,7 @@ class S3:
         resource_id: Union[str, dict],
         key: str,
         convert_to_dict: bool = True,
+        decode_response: bool = True,
     ):
         if resource in ["record"]:
             if (
@@ -183,7 +192,13 @@ class S3:
             timestamp=timestamp,
         )
         r = requests.post(url)
-        response_content = xmltodict.parse(r.text) if convert_to_dict else r.text
+        if decode_response:
+            response_text = unquote(r.text)
+        else:
+            response_text = r.text
+        response_content = (
+            xmltodict.parse(response_text) if convert_to_dict else response_text
+        )
         return r.headers, used_params, response_content
 
     def delete_object(
@@ -286,13 +301,16 @@ class S3:
             timestamp=timestamp,
         )
         r = requests.head(url)
-        return r.headers, used_params, None
+        # TODO: may need to return a status code as well
+        # for now, we'll just return it as the body
+        return r.headers, used_params, {"exists": r.status_code == 200}
 
     def list_multipart_uploads(
         self,
         resource: S3Resource,
         resource_id: Union[str, dict],
         convert_to_dict: bool = True,
+        decode_response: bool = True,
     ):
         if resource in ["record"]:
             if (
@@ -316,7 +334,13 @@ class S3:
             timestamp=timestamp,
         )
         r = requests.get(url)
-        response_content = xmltodict.parse(r.text) if convert_to_dict else r.text
+        if decode_response:
+            response_text = unquote(r.text)
+        else:
+            response_text = r.text
+        response_content = (
+            xmltodict.parse(response_text) if convert_to_dict else response_text
+        )
         return r.headers, used_params, response_content
 
     def list_object_versions(
@@ -324,6 +348,7 @@ class S3:
         resource: S3Resource,
         resource_id: Union[str, dict],
         convert_to_dict: bool = True,
+        decode_response: bool = True,
     ):
         if resource in ["record"]:
             if (
@@ -347,7 +372,13 @@ class S3:
             timestamp=timestamp,
         )
         r = requests.get(url)
-        response_content = xmltodict.parse(r.text) if convert_to_dict else r.text
+        if decode_response:
+            response_text = unquote(r.text)
+        else:
+            response_text = r.text
+        response_content = (
+            xmltodict.parse(response_text) if convert_to_dict else response_text
+        )
         return r.headers, used_params, response_content
 
     def list_objects_v2(
@@ -356,6 +387,7 @@ class S3:
         resource_id: Union[str, dict],
         continuation_token: Optional[str] = None,
         convert_to_dict: bool = True,
+        decode_response: bool = True,
     ):
         if resource in ["record"]:
             if (
@@ -381,7 +413,13 @@ class S3:
             timestamp=timestamp,
         )
         r = requests.get(url)
-        response_content = xmltodict.parse(r.text) if convert_to_dict else r.text
+        if decode_response:
+            response_text = unquote(r.text)
+        else:
+            response_text = r.text
+        response_content = (
+            xmltodict.parse(response_text) if convert_to_dict else response_text
+        )
         return r.headers, used_params, response_content
 
     def list_parts(
@@ -392,6 +430,7 @@ class S3:
         upload_id: str,
         part_number_marker: Optional[int] = None,
         convert_to_dict: bool = True,
+        decode_response: bool = True,
     ):
         if resource in ["record"]:
             if (
@@ -417,7 +456,13 @@ class S3:
             timestamp=timestamp,
         )
         r = requests.get(url)
-        response_content = xmltodict.parse(r.text) if convert_to_dict else r.text
+        if decode_response:
+            response_text = unquote(r.text)
+        else:
+            response_text = r.text
+        response_content = (
+            xmltodict.parse(response_text) if convert_to_dict else response_text
+        )
         return r.headers, used_params, response_content
 
     def put_object(
